package com.devnation.helix_timex;

import android.bluetooth.BluetoothDevice;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.recyclerview.widget.RecyclerView;

import com.crrepa.ble.scan.bean.CRPScanDevice;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;


class ScanResultsAdapter extends RecyclerView.Adapter<ScanResultsAdapter.ViewHolder> {

    static class ViewHolder extends RecyclerView.ViewHolder {

        TextView line1;
        TextView line2;

        ViewHolder(View itemView) {
            super(itemView);
//            ButterKnife.bind(this, itemView);
            line1=itemView.findViewById(android.R.id.text1);
            line2=itemView.findViewById(android.R.id.text2);
        }
    }

    interface OnAdapterItemClickListener {

        void onAdapterViewClick(View view);
    }

    private static final Comparator<CRPScanDevice> SORTING_COMPARATOR = new Comparator<CRPScanDevice>() {
        @Override
        public int compare(CRPScanDevice scanDevice, CRPScanDevice t1) {
            return scanDevice.getDevice().getAddress().compareTo(t1.getDevice().getAddress());
        }
    };

    private final List<CRPScanDevice> data = new ArrayList<>();
    private OnAdapterItemClickListener onAdapterItemClickListener;
    private final View.OnClickListener onClickListener = new View.OnClickListener() {
        @Override
        public void onClick(View v) {

            if (onAdapterItemClickListener != null) {
                onAdapterItemClickListener.onAdapterViewClick(v);
            }
        }
    };

    void addScanResult(CRPScanDevice bleScanResult) {
        for (int i = 0; i < data.size(); i++) {
            if (data.get(i).getDevice().equals(bleScanResult.getDevice())) {
                data.set(i, bleScanResult);
                notifyItemChanged(i);
                return;
            }
        }

        data.add(bleScanResult);
        Collections.sort(data, SORTING_COMPARATOR);
        notifyDataSetChanged();
    }

    void clearScanResults() {
        data.clear();
        notifyDataSetChanged();
    }

    public CRPScanDevice getItemAtPosition(int childAdapterPosition) {
        return data.get(childAdapterPosition);
    }

    @Override
    public int getItemCount() {
        return data.size();
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        final CRPScanDevice scanDevice = data.get(position);
        final BluetoothDevice bleDevice = scanDevice.getDevice();
        holder.line1.setText(String.format("%s (%s)", bleDevice.getAddress(), bleDevice.getName()));
        holder.line2.setText(String.format("RSSI: %d", scanDevice.getRssi()));
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        final View itemView = LayoutInflater.from(parent.getContext()).inflate(android.R.layout.two_line_list_item, parent, false);
        itemView.setOnClickListener(onClickListener);
        return new ViewHolder(itemView);
    }

    void setOnAdapterItemClickListener(OnAdapterItemClickListener onAdapterItemClickListener) {
        this.onAdapterItemClickListener = onAdapterItemClickListener;
    }
}
